!
      SUBROUTINE U_PERLS3(UA,UB,UC,THETA,OFSD,OFSQ,ZPD,ZPQ,ZND,ZNQ,FCS, &
     &RELBW)
! 
! Purpose - extract complex constants from 3-phase measurements such that
!
!   U(t) = (OFSD+j*OFSQ) + (ZPD+j*ZPQ)*exp(j*THETA) + conj[(ZND+j*ZNQ)*exp(j*THETA)]
!
! where THETA is a given reference funcion with frequency FCS
!
!
! Method: Stationary solution to RLS estimation of OFSD,OFSQ,ZPD,ZPQ,ZND,ZNQ
!
! Date    - 2010-09-25
! Author  - Lennart ngquist

! standard include files
      INCLUDE 'nd.h'
      INCLUDE 'emtstor.h'
      INCLUDE 's1.h'
      INCLUDE 'emtconst.h'

! variable definitions
      IMPLICIT NONE

      REAL UA,UB,UC,THETA,OFSD,OFSQ,ZPD,ZPQ,ZND,ZNQ,FCS,RELBW
      INTEGER NC
      COMPLEX J,U,EJTH,EJTHMU,OFS,ZP,ZN,UHAT,INNOV
      REAL RELBW2,KAPPAOFS,KAPPAPH,MU,WCS

! storage pointers
      NC     = NSTORC
      NSTORC = NSTORC + 3
      
! constants
      J = CMPLX(0.,1.)

! get complex input vector
      U = (2.0*UA-UB-UC)/3.0 + J*(UB-UC)/SQRT(3.0)
       
! get auxiliary constants
      RELBW2   = RELBW*RELBW
      KAPPAOFS = RELBW*(1+RELBW2)
      KAPPAPH  = RELBW*SQRT(1 + 1.25*RELBW2 + 0.25*RELBW2*RELBW2)
      MU       = ATAN2(3*RELBW,2-RELBW2)
      WCS      = TWO_PI*FCS

      EJTH     = CEXP(J*THETA)
      EJTHMU   = CEXP(-J*(THETA+MU))

! update estimations
      OFS   = STORC(NC  )
      ZP    = STORC(NC+1)
      ZN    = STORC(NC+2)

      UHAT  = OFS + ZP*EJTH + CONJG(ZN*EJTH)
      INNOV = U - UHAT

      OFS   = OFS + WCS*KAPPAOFS*DELT*INNOV
      ZP    = ZP  + WCS*KAPPAPH*DELT*INNOV*EJTHMU
      ZN    = ZN  + WCS*KAPPAPH*DELT*CONJG(INNOV)*EJTHMU
             
! update storage
      STORC(NC  ) = OFS
      STORC(NC+1) = ZP
      STORC(NC+2) = ZN

! put output variables
      OFSD = REAL(OFS)
      OFSQ = AIMAG(OFS)
      ZPD  = REAL(ZP)
      ZPQ  = AIMAG(ZP)
      ZND  = REAL(ZN)
      ZNQ  = AIMAG(ZN)

      RETURN
      END
