!

      SUBROUTINE U_PLLRLS3(UA,UB,UC,OFSD,OFSQ,ZPD,ZPQ,ZND,ZNQ,FCS,      &
     &THETA,FCS0,RELBW,KPLL,TPLL)
!
! Purpose - phase-locked loop that locks on the positive sequence component,
!           Extracts complex constants from 3-phase measurements such that
!
!           U(t) = (OFSD+j*OFSQ) + (ZPD+j*ZPQ)*exp(j*THETA) + conj[(ZND+j*ZNQ)*exp(j*THETA)]
!
!           where THETA is a given reference funcion with frequency FCS
!
!
! Method  - Stationary solution to RLS estimation of OFSD,OFSQ,ZPD,ZPQ,ZND,ZNQ
!        
!
! Date    - 2010-09-26
! Author  - Lennart ngquist

! standard include files
      INCLUDE 'nd.h'
      INCLUDE 'emtstor.h'
      INCLUDE 's1.h'
      INCLUDE 'emtconst.h'

! variable definitions
      IMPLICIT NONE

      REAL UA,UB,UC,OFSD,OFSQ,ZPD,ZPQ,ZND,ZNQ,FCS,THETA
      REAL FCS0,RELBW,KPLL,TPLL
      INTEGER NF,NC
      COMPLEX J,U,EJTH,EJTHMU,OFS,ZP,ZN,UHAT,INNOV
      REAL RELBW2,KAPPAOFS,KAPPAPH,MU,WCS,ARGZP,DFCSI

! storage pointers
      NF     = NSTORF
      NSTORF = NF + 2
      NC     = NSTORC
      NSTORC = NC + 3
      
! constants
      J = CMPLX(0.,1.)

! get actual argument
      THETA = STORF(NF  )

! get complex input vector
      U = (2.0*UA-UB-UC)/3.0 + J*(UB-UC)/SQRT(3.0)
       
! get auxiliary constants
      RELBW2   = RELBW*RELBW
      KAPPAOFS = RELBW*(1+RELBW2)
      KAPPAPH  = RELBW*SQRT(1 + 1.25*RELBW2 + 0.25*RELBW2*RELBW2)
      MU       = ATAN2(3*RELBW,2-RELBW2)
      WCS      = TWO_PI*FCS

      EJTH     = CEXP(J*THETA)
      EJTHMU   = CEXP(-J*(THETA+MU))

! update estimations
      OFS   = STORC(NC  )
      ZP    = STORC(NC+1)
      ZN    = STORC(NC+2)

      UHAT  = OFS + ZP*EJTH + CONJG(ZN*EJTH)
      INNOV = U - UHAT

      OFS   = OFS + WCS*KAPPAOFS*DELT*INNOV
      ZP    = ZP  + WCS*KAPPAPH*DELT*INNOV*EJTHMU
      ZN    = ZN  + WCS*KAPPAPH*DELT*CONJG(INNOV)*EJTHMU

! update frequency controller
      IF ( ABS(ZP) .NE. 0 ) THEN
        ARGZP = ATAN2(AIMAG(ZP),REAL(ZP))
      ELSE
        ARGZP = 0
      ENDIF

      DFCSI = STORF(NF+1)
      DFCSI = DFCSI + KPLL*DELT/TPLL*ARGZP
      FCS   = FCS0 + DFCSI + KPLL*ARGZP
      THETA = THETA + TWO_PI*FCS*DELT
      THETA = MODULO(THETA,TWO_PI)
                   
! update storage
      STORF(NF  ) = THETA
      STORF(NF+1) = DFCSI

      STORC(NC  ) = OFS
      STORC(NC+1) = ZP
      STORC(NC+2) = ZN

! put output variables
      OFSD = REAL(OFS)
      OFSQ = AIMAG(OFS)
      ZPD  = REAL(ZP)
      ZPQ  = AIMAG(ZP)
      ZND  = REAL(ZN)
      ZNQ  = AIMAG(ZN)

      RETURN
      END
